
# coding: utf-8

# # Basic setup and data (with plot)

# In[1]:

get_ipython().magic(u'matplotlib inline')

import csv
import requests
import ggplot
import matplotlib
matplotlib.use('agg')
matplotlib.style.use('ggplot')
import pandas as pd
import matplotlib.pyplot as plt
import statsmodels.api as sm
import numpy as np
from statsmodels.tsa.arima_process import ArmaProcess
from StringIO import StringIO

# need ssh tunnel to use this url
url = 'http://zope:runescape@localhost:2200/erp5/portal_skins/erp5_r_order_prediction_tutorial/SaleOrderModule_getArimaPredictionList'
response = requests.get(url, auth=('zope', 'runescape'))
csv_file = StringIO()
csv_file.write(response.text)
csv_file.seek(0)

ts = pd.read_csv(csv_file, sep=';',
                 parse_dates=[0],
                 infer_datetime_format=True,
                 squeeze=True,
                 index_col='date')
ts.plot()


# # Graphics to analyze auto correlation and partial auto correlation

# In[2]:

fig = plt.figure(figsize=(12, 8))
ax1 = fig.add_subplot(211)
fig = sm.graphics.tsa.plot_acf(ts, lags=30, ax=ax1)

ax2 = fig.add_subplot(212)
fig = sm.graphics.tsa.plot_pacf(ts, lags=30, ax=ax2)


# Looks like partial correlation going to 0 after lag 3. So **ARMA(3,0)** might be  an adequated model for out data.

# # Minimum order selection for ARMA

# In[28]:

answers = [sm.tsa.arma_order_select_ic(ts, ic=['aic'], trend=trend, fit_kw={'method': 'css'}) for trend in ['c', 'nc']]
for answer in answers:
    print answer['aic_min_order']


# # Check if series is stationary

# Using the Augmented Dicker-Fuller test we can check if out data is stationary. For this, we need to get an **adf statistic** value __smaller__ (more negative) than the critical value at 5% for regression **c**, **ct** and **nc**.

# In[15]:

def check_stationarity(series, regression='c'):
    regressions = {
        'c':   'constant only',
        'ct':  'constant and trend',
        'ctt': 'constant, linear and quadratic trend',
        'nc':  'no constant, no trend'
    }
    result = sm.tsa.adfuller(series, regression=regression, autolag='t-stat', store=True)
    store = result[-1]
    if store.adfstat > result[2]['5%']:
        print 'Time series is not stationary with %s. See data below:' % regressions[regression]
        print '  - adf statistics: ', store.adfstat
        print '  - p-value: ', result[1]
        print '  - critical values: ', store.critvalues
        return False
    else:
        print 'Time series is stationary with regression %s! :D\n' % regressions[regression]
        return True

stationarity_tests = [check_stationarity(ts, regression) for regression in ['c', 'ct', 'ctt', 'nc']]
diff_ts = ts.diff()
diff_ts.plot()
    


# # Fit the ARIMA model and plot the data + forecast

# In[32]:

arima = sm.tsa.ARIMA(ts, (3,0,0))
arima_fit = arima.fit()

arima_fit.resid.plot()

fig, ax = plt.subplots()
ax = ts.plot(ax=ax)
fig = arima_fit.plot_predict(100, 200, dynamic=True, ax=ax)
plt.show()

